﻿//*************************************************}
//                                                 }
//       Class: Tectus HF-Reader                   }
//       Programmed with Visual Studio 2008        }
//       Tectus Transponder Technology GmbH        }
//       Sascha Haveloh, s.haveloh@tec-tus.de      }
//                                                 }
//*************************************************}
// Version 1.0 
// Visual Studio 2008
// 14.01.2013
// 21.03.2013..ClearComBuffer


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;



public class RFIDTectusHF : Form 
{
    public RFIDTectusHF()
    {
      Init();
    }

    ~RFIDTectusHF()
    {
        DeInit();
    }

    public sealed class ControlChars
    {
        public const char Back = '\b';
        public const char Cr = '\r';
        public const string CrLf = "\r\n";
        public const char FormFeed = '\f';
        public const char Lf = '\n';
        public const string NewLine = "\r\n";
        public const char NullChar = '\0';
        public const char Quote = '"';
        public const char Tab = '\t';
        public const char VerticalTab = '\v';
     }


    #region "Variablen"
    private string _Version = "V1.0";

    private string fLastException = string.Empty;  
    private string scanline = string.Empty;
    private string fReceiveStr = string.Empty;
    private int HFCommand = -1;
    private int fErrorID = -1;
    private string fErrorText = string.Empty; 
    private string fTyp_iso = string.Empty; 
    private string fUID_iso = string.Empty; 
    private string fDSF_iso = string.Empty; 
    private string fTiF_iso = string.Empty; 
    private string fUIDFeig_iso = string.Empty;

    private string fTyp_mif = string.Empty;
    private string fUID_mif = string.Empty;
    private string fUIDFeig_mif = string.Empty;

    //[Typ]:010000 (ISO-15693) [UID]:74474101000005E0 [DSF]:00 [TiF]:00"... Command id
    //[Typ]:004400 (MIFARE Ultralight) [UID]:04D5F97A551D8000"



    private System.IO.Ports.SerialPort SerialPort1 = new System.IO.Ports.SerialPort();
    private System.Windows.Forms.Timer DelayTimer = new System.Windows.Forms.Timer();

    #endregion

    #region "Propertys"

    public string Typ_ISO
    {
        get { return fTyp_iso; }
    }

    public string UIDFeigNr_ISO  // MC9090 compatible
    {
        get { return fUIDFeig_iso; }
    }

    public string UIDNr_ISO
    {
        get { return fUID_iso; }
    }

    public string DSF_ISO
    {
        get { return fDSF_iso; }
    }

    public string TiF_ISO
    {
        get { return fTiF_iso; }
    }


    public string Typ_MIF
    {
        get { return fTyp_mif; }
    }

    public string UIDFeigNr_MIF  // MC9090 compatible
    {
        get { return fUIDFeig_mif; }
    }

    public string UIDNr_MIF
    {
        get { return fUID_mif; }
    }


    public string DLLVersion
    {
        get { return _Version; }
    }

    public string LastException
    {
        get { return fLastException; } 
    }

    public int ErrorID
    {
        get { return fErrorID; } 
    }

    public string ErrorText
    {
        get { return fErrorText; }
    }

    public int TimeOut
    {
        get { return DelayTimer.Interval; }
        set { DelayTimer.Interval = value; }
    }

    public bool CTSInfo
    {
        get { return SerialPort1.CtsHolding; }
    }

    public bool CDInfo
    {
        get { return SerialPort1.CDHolding; }
    }

    public bool DSRInfo
    {
        get { return SerialPort1.DsrHolding; }
    }

    public bool DTR
    {
        get { return SerialPort1.DtrEnable; }
        set { SerialPort1.DtrEnable = value; }
    }

    public bool RTS
    {
        get { return SerialPort1.RtsEnable; }
        set { SerialPort1.RtsEnable = value; }
    }

    public System.IO.Ports.StopBits StopBits
    {
        get { return SerialPort1.StopBits; }
        set { SerialPort1.StopBits = value; }
    }

    public int DataBits
    {
        get { return SerialPort1.DataBits; }
        set { SerialPort1.DataBits = value; }
    }

    public System.IO.Ports.Handshake Handshake
    {
        get { return SerialPort1.Handshake; }
        set { SerialPort1.Handshake = value; }
    }

    public System.IO.Ports.Parity Parity
    {
        get { return SerialPort1.Parity; }
        set { SerialPort1.Parity = value; }
    }

    public int BaudRate
    {
        get { return SerialPort1.BaudRate; }
        set { SerialPort1.BaudRate = value; }
    }

    public string Comport
    {
        get { return SerialPort1.PortName; }
        set { SerialPort1.PortName = value; }
    }

    public int WriteTimeOut
    {
        get { return SerialPort1.WriteTimeout; }
        set { SerialPort1.WriteTimeout = value; }
    }

    public int ReadTimeOut
    {
        get { return SerialPort1.ReadTimeout; }
        set { SerialPort1.ReadTimeout = value; }
    }

    public bool Active
    {
        get { return SerialPort1.IsOpen;}
        set
        {
            if (value)
            {
                try
                {
                    SerialPort1.ReadTimeout = 1000;
                    SerialPort1.WriteTimeout = 1000;
                    SerialPort1.Open();
                    SerialPort1.DtrEnable = true;
                    SerialPort1.RtsEnable = true;
                }
                catch (Exception ex)
                {
                    fLastException = ex.Message.ToString();
                }
            }
            else
            {
                try
                {
                    SerialPort1.DtrEnable = false;
                    SerialPort1.RtsEnable = false;
                    SerialPort1.Close();
                }
                catch (Exception ex)
                {
                    fLastException = ex.Message.ToString();
                }
            }
        }
    }


    #endregion

    #region "Events"
    public event GetHFDataEventHandler HFData;
    public delegate void GetHFDataEventHandler(object sender, string Direction, string Data);

    #endregion

    #region "AddEvents"
    private void AddEvents()
    {
        this.SerialPort1.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(this.SerialPort1_DataReceived);
        this.DelayTimer.Tick += new System.EventHandler(this.DelayTimer_Tick);
    }

    #endregion

    #region "Function/Void"
    public string GetVersion()
    {
        string Result = "";
        HFCommand = 1; // Command for Software version
        if (SendTagString("sv"))
        {
            Result = fReceiveStr;
        }
        return Result;
    }

    public bool GetUID_MIF()
    {
        bool result = false;
        int p = -1;
        HFCommand = 3; // Command for Identifity Mifare
        ClearVariable_MIF();
        if (SendTagString("md"))
        {
            result = (ErrorID == 0);
        }
        return result;
    }

    public bool GetUID_ISO()
    {
        bool result = false;
        int p = -1;
        HFCommand = 2; // Command for Identifity ISO15693
        ClearVariable_ISO();
        if (SendTagString("id"))
        {
            result = (ErrorID == 0);
        }
        return result;
    }

    private void ClearVariable_MIF()
    {
        fTyp_mif = "";
        fUID_mif = "";
        fUIDFeig_mif = "";
        fErrorID = -1;
    }

    private void ClearVariable_ISO()
    {
        fTyp_iso = "";
        fUID_iso = "";
        fDSF_iso = "";
        fTiF_iso = "";
        fUIDFeig_iso = "";
        fErrorID = -1;
    }

    private void ClearComBuffer()
    {
        if (SerialPort1.IsOpen)
        {
            SerialPort1.DiscardInBuffer();
            SerialPort1.DiscardOutBuffer();
        }
    }

    private void SerialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
    {
        scanline += SerialPort1.ReadExisting();
        if (scanline.IndexOf(ControlChars.Cr) >= 0)
        {
            scanline = scanline.Replace("\r", "");
            if (scanline.Length > 0)
            {
                this.Invoke(new EventHandler(MyDataReceivedAll));
            }
        }
    }

    private void DelayTimer_Tick(object sender, System.EventArgs e)
    {
        DelayTimer.Enabled = false;
    }


    private void SetReaderText(string Direction, string data)
    {
        if (HFData != null) { HFData(this, Direction, data); }
    }

    private void MyDataReceivedAll(object sender, System.EventArgs e)
    {
        SetReaderText("<", scanline);
        ComRecNeu(scanline);
    }

    private void ComRecNeu(string RecBuff)
    {
        string tmpUID = string.Empty;
        int p1, p2, p3, p4;
        if (RecBuff.Length > 0)
        {
            DelayTimer.Enabled = false;
            fReceiveStr = RecBuff;
            CheckErrorID(fReceiveStr);
            if (HFCommand == 2) //[Typ]:010000 (ISO-15693) [UID]:74474101000005E0 [DSF]:00 [TiF]:00"... Command id..ISO
            {
                p1 = fReceiveStr.IndexOf("[Typ]:");
                p2 = fReceiveStr.IndexOf("[UID]:");
                p3 = fReceiveStr.IndexOf("[DSF]:");
                p4 = fReceiveStr.IndexOf("[TiF]:");
                if ((p1 >= 0) & (p2 >= 0) & (p3 >= 0) &(p4 >= 0))
                {
                    fTyp_iso = fReceiveStr.Substring(6, p2 - 6).Trim();
                    fUID_iso = fReceiveStr.Substring(p2 + 6, p3 - p2 - 6).Trim();
                    fDSF_iso = fReceiveStr.Substring(p3 + 6, p4 - p3 - 6).Trim();
                    fTiF_iso = fReceiveStr.Substring(p4 + 6).Trim();
                    fUIDFeig_iso = GetFeigNr_ISO(fUID_iso);
                    fErrorID = 0;
                }
            }
            if (HFCommand == 3)  // "[Typ]:004400 (MIFARE Ultralight) [UID]:04D5F97A551D8000"
            {
                p1 = fReceiveStr.IndexOf("[Typ]:");
                p2 = fReceiveStr.IndexOf("[UID]:");
                if ((p1 >= 0) & (p2 >= 0))
                {
                    fTyp_mif = fReceiveStr.Substring(6, p2 - 6).Trim();
                    fUID_mif = fReceiveStr.Substring(p2 + 6);
                    fUIDFeig_mif = GetFeigNr_MIF(fUID_mif);  
                    fErrorID = 0;                    
                }

            }
            if (HFCommand == 10)  // read iso
            {
                // dummy
            }
            if (HFCommand == 11) // write iso
            {
                // dummy
            }
            if (HFCommand == 20) // read mif
            {
                // dummy
            }
            if (HFCommand == 21) // write mif
            {
                // dummy
            }

        }
     }

    private void CheckErrorID(string Input)
    {
        int p;
        p = Input.IndexOf("[ERR]:");
        if (p >= 0)
        {
            fErrorID = Convert.ToInt32(Input.Substring(6));
            fErrorText = Input;
        }
    }


    // Tag-Info Test Tags...ISO 15693...   
    // Tectus Number  74474101000005E0
    // Feig   Number  E005000001414774

    // Tectus Number 04D5F97A551D8000
    // Feig   Number 00801D557AF9D504
    


    private string GetFeigNr_ISO(string tmpUID)
    {
        string resultUID = "";
        if (tmpUID.Length >= 16)
        {
            //74474101000005E0..to E005000001414774
            resultUID = tmpUID.Substring(14, 2) + 
                        tmpUID.Substring(12, 2) + 
                        tmpUID.Substring(10, 2) + 
                        tmpUID.Substring(8, 2) + 
                        tmpUID.Substring(6, 2) + 
                        tmpUID.Substring(4, 2) + 
                        tmpUID.Substring(2, 2) + 
                        tmpUID.Substring(0, 2);
        }
        return resultUID;
    }

    private string GetFeigNr_MIF(string tmpUID)
    {
        string resultUID = "";
        if (tmpUID.Length >= 16)
        {
            //04D5F97A551D8000..to 00801D557AF9D504
            resultUID = tmpUID.Substring(14, 2) +
                        tmpUID.Substring(12, 2) +
                        tmpUID.Substring(10, 2) +
                        tmpUID.Substring(8, 2) +
                        tmpUID.Substring(6, 2) +
                        tmpUID.Substring(4, 2) +
                        tmpUID.Substring(2, 2) +
                        tmpUID.Substring(0, 2);
        }
        return resultUID;
    }

    public string GetSplitBlockData(string BlockNr, string BlockLength, string BlockData)
    {

        // "[B02]:54656374 [B03]:75732048 [B04]:46205265 [B05]:61646572 " Result from reader
        // Split = "54656374757320484620526561646572"
        // Example 


        // write ASCII = "Tectus HF Reader" 
        // change to Hex = "54656374757320484620526561646572"
        // write 
        // "iw[02,04]54656374757320484620526561646572"
        // read 
        // "ir[02,04]


        string spltdata = "";
        int Start = Convert.ToInt32(BlockNr, 16); // HexToInt
        int Len = Convert.ToInt32(BlockLength, 16); // HexToInt
        int Stop = Start + Len;
        int pos;
        for (int i = Start; i <= Stop; i++)
        {
            pos = BlockData.IndexOf("[B" + i.ToString("D2") + "]:");
            if (pos >= 0)
            {
                spltdata += BlockData.Substring(pos + 6, 8);
            }
        }
        return spltdata;
    }

    public string WriteTagBlock_ISO(string BlockNr, string BlockLength, string Blockdata)
    {
        string result = string.Empty;
        HFCommand = 11;  // write command
        if (SendTagString("iw[" + BlockNr + "," + BlockLength + "]"+Blockdata))
            result = fReceiveStr;
        return result;
    }

    public string ReadTagBlock_ISO(string BlockNr, string BlockLength)
    {
        string result = string.Empty;
        HFCommand = 10;  // read command
        if (SendTagString("ir[" + BlockNr + "," + BlockLength + "]"))
            result = fReceiveStr; 
        return result;
    }

    public string WriteTagBlock_MIF(string BlockNr, string BlockLength, string Blockdata)
    {
        string result = string.Empty;
        HFCommand = 21;  // write command
        if (SendTagString("mw[" + BlockNr + "," + BlockLength + "]" + Blockdata))
            result = fReceiveStr;
        return result;
    }

    public string ReadTagBlock_MIF(string BlockNr, string BlockLength)
    {
        string result = string.Empty;
        HFCommand = 20;  // read command
        if (SendTagString("mr[" + BlockNr + "," + BlockLength + "]"))
            result = fReceiveStr;
        return result;
    }

    private void SendFData_COM(char[] RData, int len)
    {
        //for (int i = 0; i <= len; i++) {SerialPort1.Write(RData, i, 1);}
        SerialPort1.Write(RData, 0, RData.Length);
    }

    private bool SendTagString(string RData)
    {
        bool Result = false;
        if (this.Active != true) { return Result;}
        ClearComBuffer();
        SetReaderText(">", RData);
        fReceiveStr = "";
        scanline = "";
        RData = RData + ControlChars.Cr;
        SendFData_COM(RData.ToCharArray(), RData.Length - 1);
        DelayTimer.Enabled = true;
        while (DelayTimer.Enabled)
        {
            System.Windows.Forms.Application.DoEvents();
        }
        Result = !string.IsNullOrEmpty(fReceiveStr);
        return Result;
    }

    public string SendDataToReader(string Command)
    {
        string Result = "";
        HFCommand = 99;
        if (SendTagString(Command))
        {
            Result = fReceiveStr;
        }
        return Result;
    }


    private void Init()
    {
        //this.CreateHandle(); // .. Muss gestartet werden, sonst handle fehler
        //this.CreateHandle(); 
        AddEvents();
        SerialPort1.BaudRate = 9600;
        SerialPort1.DataBits = 8;
        SerialPort1.Parity = System.IO.Ports.Parity.None;
        SerialPort1.StopBits = System.IO.Ports.StopBits.One;
        SerialPort1.DtrEnable = false;
        SerialPort1.RtsEnable = false;
        SerialPort1.PortName = "COM1";
        DelayTimer.Interval = 2000;
    }

    private void DeInit()
    {
        DelayTimer.Enabled = false;
        this.Active = false;
        SerialPort1.Dispose();
        DelayTimer.Dispose();
        SerialPort1 = null;
        DelayTimer = null;
    }


    #endregion

 
    #region "utility"


    private byte[] ASCIIToByteArray(string str)
    {
        byte[] Result;
        ASCIIEncoding enc = new System.Text.ASCIIEncoding();
        Result = enc.GetBytes(str);
        return Result;
    }

    public string ByteArrayToASCII(byte[] arr)
    {
        string Result = string.Empty;
        System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
        Result = enc.GetString(arr, 0, arr.Length);
        return Result;
    }


    public void ClearStringArray(ref string[] src)
    {
        for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++)
        {
            src[i] = "";
        }
    }

    public void ClearCharArray(ref char[] src)
    {
        for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++)
        {
            src[i] = '\0';
        }
    }

    public void ClearByteArray(ref byte[] src)
    {
        for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++)
        {
            src[i] = 0;
        }
        // Array.Clear(src, 0, src.Length);  another solution
        // for (int i; i = 0; i <= src.length) { src[i] = 0; }
    }

    public string GetInverted(string Data)
    {
        string Result = string.Empty;
        for (int i = Data.Length - 1; i >= 0; i += -1)
        {
            Result = Result + Data[i];
        }
        return Result;
    }


    public string GetByteDataBufferToHex(byte[] Src, int BStart, int BStop)
    {
        string Result = string.Empty;
        for (int i = BStart; i <= BStop; i++)
        {
            Result += Src[i].ToString("X2");
        }
        return Result;
    }

    public string GetByteDataBufferToHexInverted(byte[] Src, int BStart, int BStop)
    {
        string Result = string.Empty;
        for (int i = BStop; i >= BStart; i += -1)
        {
            Result += Src[i].ToString("X2");
        }
        return Result;
    }

    public string GetByteBufferToHex(byte[] Src, int DataLen)
    {
        string Result = string.Empty;
        for (int i = 0; i <= DataLen - 1; i++)
        {
            Result += Src[i].ToString("X2");
        }
        return Result;
    }

    public string ASCIIToHex(string Input)
    {
        string Result = string.Empty;
        System.Text.StringBuilder sb = new System.Text.StringBuilder();
        foreach (char c in Input)
        {
            sb.Append(Convert.ToInt32(c).ToString("X2"));
        }
        Result = sb.ToString();
        return Result;
    }

    public string HexToASCII(string Input)
    {
        string Result = string.Empty;
        System.Text.StringBuilder sb = new System.Text.StringBuilder();
        for (int i = 0; i <= Input.Length - 2; i += 2)
        {
            sb.Append(Convert.ToChar(Convert.ToByte(Input.Substring(i, 2), 16))).ToString();
        }
        Result = sb.ToString();
        return Result;
    }

    public int HexToInt(string Input)
    {
        return Convert.ToInt32(Input, 16);
    }


    public byte[] HexToByteArray(string input)
    {
        byte[] result;
        int c = 0;
        result = new byte[input.Length / 2];
        for (int i = 0; i <= input.Length - 1; i += 2)
        {
            result[c] = Convert.ToByte(input.Substring(i, 2).ToString(), 16);
            c += 1;
        }
        return result;
    }

    public string ByteToHex(byte[] src)
    {
        string Result = string.Empty;
        for (int i = src.GetLowerBound(0); i <= src.GetUpperBound(0); i++)
        {
            Result += src[i].ToString("X2");
        }
        return Result;
    }

    public byte[] GetASCIIByte(string src)
    {
        byte[] Result;
        ASCIIEncoding enc = new System.Text.ASCIIEncoding();
        Result = enc.GetBytes(src);
        return Result;
    }

    public byte[] GetASCIIByteFix(string src)
    {
        byte[] Result = new byte[256];
        for (int i = 0; i <= src.Length - 1; i++)
        {
            Result[i] = (byte)Convert.ToChar(src.Substring(i, 1));
        }
        return Result;
    }

    public char[] GetASCIIChar(string src)
    {
        char[] Result = new char[src.Length - 1];
        Result = src.ToCharArray();
        return Result;
    }

    #endregion

    private void InitializeComponent()
    {
        this.SuspendLayout();
        // 
        // RFIDTectusHF
        // 
        this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Inherit;
        this.ClientSize = new System.Drawing.Size(240, 294);
        this.Name = "RFIDTectusHF";
        this.ResumeLayout(false);

    }


}
